@@ -1,7 +1,6 @@
 # Copyright © 2021 Chris Hughes
 import math
 import os
-
 from enum import Enum
 from functools import partial
 from typing import Iterable
@@ -13,23 +12,22 @@ from torch.utils.data import DataLoader
 
 from pytorch_accelerated.callbacks import (
     CallbackHandler,
-    LogMetricsCallback,
-    PrintProgressCallback,
-    TerminateOnNaNCallback,
-    StopTrainingError,
-    ProgressBarCallback,
-    MoveModulesToDeviceCallback,
     LimitBatchesCallback,
+    LogMetricsCallback,
+    MoveModulesToDeviceCallback,
+    PrintProgressCallback,
+    ProgressBarCallback,
+    StopTrainingError,
+    TerminateOnNaNCallback,
 )
 from pytorch_accelerated.run_config import TrainerRunConfig
-from pytorch_accelerated.tracking import RunHistory, InMemoryRunHistory, LossTracker
+from pytorch_accelerated.tracking import InMemoryRunHistory, LossTracker, RunHistory
 from pytorch_accelerated.utils import (
     LIMIT_BATCHES_ENV_VAR,
-    worker_init_fn,
     remove_padding,
+    worker_init_fn,
 )
 
-
 DEFAULT_CALLBACKS = (
     MoveModulesToDeviceCallback,
     TerminateOnNaNCallback,
@@ -161,6 +159,7 @@ class Trainer:
         )
         self._accelerator = self._create_accelerator()
         self._loss_tracker = LossTracker()
+        self._pad_uneven_batches = False
         # placeholders which will be set during training
         self.create_scheduler_fn = None
         self.scheduler = None
@@ -384,7 +383,7 @@ class Trainer:
         """
         This method is called at the end of a training run.
         """
-        pass
+        self.model = self.get_model()
 
     def evaluation_run_start(self):
         """
@@ -396,7 +395,7 @@ class Trainer:
         """
         This method is called at the end of an evaluation run.
         """
-        pass
+        self.model = self.get_model()
 
     def train(
         self,
@@ -761,7 +760,7 @@ class Trainer:
                 self.optimizer_zero_grad()
 
         self.train_epoch_end()
-        self.run_history.update_metric("train_loss_epoch", self._loss_tracker.average)
+        self._add_epoch_loss_to_run_history("train_loss_epoch")
         self.callback_handler.call_event(
             "on_train_epoch_end",
             self,
@@ -777,16 +776,40 @@ class Trainer:
         if self.run_config.gradient_accumulation_steps > 1:
             batch_output["loss"] /= self.run_config.gradient_accumulation_steps
 
-        self._loss_tracker.update(
-            self.gather(batch_output["loss"]).detach().mean().item(),
-            batch_output["batch_size"],
-        )
+        self._update_loss_tracker(batch_output["loss"], batch_output["batch_size"])
 
         self.callback_handler.call_event(
             "on_train_step_end", self, batch_output=batch_output, batch=batch
         )
         self.backward_step(batch_output["loss"])
 
+    def _update_loss_tracker(self, batch_loss, batch_size):
+        """
+        Update the loss calculated for each batch using the internal loss tracker.
+        During each epoch, losses are tracked in individual processes.
+        """
+        self._loss_tracker.update(
+            batch_loss.detach().mean().item(),
+            batch_size,
+        )
+
+    def _add_epoch_loss_to_run_history(self, metric_name):
+        """
+        Update the run history with the average of all batch losses calculated during the epoch across all processes.
+        """
+        total_loss_per_process = torch.tensor(
+            self._loss_tracker.total_loss, device=self.device
+        )
+        running_count_per_process = torch.tensor(
+            self._loss_tracker.running_count, device=self.device
+        )
+
+        total_loss = self.gather(total_loss_per_process)
+        running_count = self.gather(running_count_per_process)
+
+        average_loss = total_loss.sum() / running_count.sum()
+        self.run_history.update_metric(metric_name, average_loss.item())
+
     def _clip_gradients(self):
         """
         Clip the gradients of the model's parameters that fall outside of the threshold specified in :meth:`~Trainer.train`.
@@ -811,22 +834,31 @@ class Trainer:
             self,
         )
 
-        for batch in valid_dl:
-            self.callback_handler.call_event(
-                "on_eval_step_start",
-                self,
-            )
-            batch_output = self.calculate_eval_batch_loss(batch)
-            self._loss_tracker.update(
-                self.gather(batch_output["loss"]).detach().mean().item(),
-                batch_output["batch_size"],
-            )
-            self.callback_handler.call_event(
-                "on_eval_step_end", self, batch_output=batch_output, batch=batch
-            )
+        # as no gradients are calculated, we do not need to sync model parameters during evaluation
+        with self._accelerator.no_sync(self.model):
+            # handle uneven batch sizes during distributed evaluation
+            with self._accelerator.join_uneven_inputs(
+                [self.model], even_batches=self._pad_uneven_batches
+            ):
+                for batch in valid_dl:
+                    self.callback_handler.call_event(
+                        "on_eval_step_start",
+                        self,
+                    )
+                    batch_output = self.calculate_eval_batch_loss(batch)
+
+                    self._update_loss_tracker(
+                        batch_output["loss"], batch_output["batch_size"]
+                    )
+
+                    self.callback_handler.call_event(
+                        "on_eval_step_end", self, batch_output=batch_output, batch=batch
+                    )
+
         self.eval_epoch_end()
         metric_name = "eval_loss_epoch" if is_training else "evaluation_loss"
-        self.run_history.update_metric(metric_name, self._loss_tracker.average)
+        self._add_epoch_loss_to_run_history(metric_name)
+
         self.callback_handler.call_event(
             "on_eval_epoch_end",
             self,
